/**
% AUTHOR: Filippo Piccinini (Email: filippo.piccinini85@gmail.com)
%
% DATE: December 15, 2020
%
% NAME: ImageJMacro_ForegroundSegmentation (version 1.00)
%
% DESCRIPTION: This Macro was created to segment the foreground 
%               of input monochannel 8-bit images.
%
% PARAMETERS:
%
% 	dir1    	Absolute path of the folder containing the monochannel
%               8-bit images to be analysed.
%
% 	dir1    	Absolute path of the folder where to save the monochannel
%               binary masks of the foreground.

% Copyright (C) 2020 Filippo Piccinini 
% IRCCS Istituto Romagnolo per lo Studio dei Tumori "Dino Amadori" (IRST)
% Meldola (FC), Italy. All rights reserved.
% Email: filippo.piccinini85@gmail.com
%
% This program is free software; you can redistribute it and/or modify it 
% under the terms of the GNU General Public License version 3 (or higher) 
% as published by the Free Software Foundation. This program is 
% distributed WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
% General Public License for more details.
**/


//Define directories
dir1 = getDirectory("Choose Source Directory ");
dir2 = getDirectory("Choose Destination Directory ");
minThreshold = 10;  //Min intensity value considered as foreground
maxThreshold = 255; //Max intensity value considered as foreground
iterN = 5; //Number of iteration for the erosion/dilation
particlePixels = 2000;//Objects with number of pixels < of this parameter will be removed from the final masks

//For each file
list = getFileList(dir1);
setBatchMode(true);
for (i=0; i<list.length; i++) {

	showProgress(i+1, list.length);
	open(dir1+list[i]);
	imgName=getTitle();

	// Global thresholding
	setThreshold(minThreshold, maxThreshold);
	setOption("BlackBackground", true);
	run("Convert to Mask");

	// Morphological operations
	for (j=0; j<iterN; j++) {
		run("Erode");
	}
	for (j=0; j<iterN; j++) {
		run("Dilate");
	}
	for (j=0; j<iterN; j++) {
		run("Erode");
	}
	for (j=0; j<iterN; j++) {
		run("Dilate");
	}

	// Small particles remotion
	run("Set Scale...", "distance=0.00 known=0.00 unit=pixel");
	run("Analyze Particles...","size=0-" + particlePixels + " circularity=0.00-1.00 show=Masks");
	maskName=getTitle();

	// Binarization to values 0 background 1 foreground
	run("Invert");
	run("Divide...", "value=255.000");
	imageCalculator("Multiply create", imgName, maskName);

	// Mask saving
	selectWindow(maskName);
	close();
	selectWindow(imgName);
	close();
	selectWindow("Result of " + imgName);
	saveAs("TIFF", dir2 + imgName);
	close();

}
setBatchMode(false);
